Typer = {}

local symbols = { '@', '#', '$', '%', '!' }
local function getRandomSymbol()
	return symbols[love.math.random(#symbols)]
end

-- for precise randomness
local function random(min, max)
	return love.math.random() * (max - min) + min
end

-- http://snippets.luacode.org/snippets/Shuffle_array_145
local function shuffle(tab)
	local n, order, res = #tab, {}, {}
	 
	for i=1,n do order[i] = { rnd = love.math.random(), idx = i } end
	table.sort(order, function(a,b) return a.rnd < b.rnd end)
	for i=1,n do res[i] = tab[order[i].idx] end
	return res
end

function Typer:initialize()
	self.font = love.graphics.newFont('font.ttf', 8)
	self.lines = {}
	self.evil = {}
	self.remorse = {}
	self.currentLine = 1
	self.currentLetter = 1
	self.flash = 0
	self.flashColor = { 255, 255, 255 }

	local random = false
	local scramble = false
	local evil = false
	local remorse = false
	for line in love.filesystem.lines('lines.txt') do
		if #line > 0 then
			if line == "#randomon" then
				random = true
			elseif line == '#randomoff' then
				random = false
			elseif line == '#scrambleon' then
				scramble = true
			elseif line == '#scrambleoff' then
				scramble = false
			elseif line == '#evil' then
				evil = true
			elseif line == '#noevil' then
				evil = false
			elseif line == '#remorse' then
				remorse = true
			elseif line == '#noremorse' then
				remorse = false
			else
				if random then
					local screwup = getRandomSymbol()
					local position = love.math.random(#line)
					line = line:gsub('()(.)', function(pos, char)
						if pos == position then
							return screwup
						else
							return char
						end
					end)
				end

				if scramble then
					local characters = {}
					for i=1, #line do
						table.insert(characters, line:sub(i,i))
					end
					line = table.concat(shuffle(characters))
				end

				table.insert(self.lines, line)

				if evil then
					self.evil[#self.lines] = true
				end
				if remorse then
					self.remorse[#self.lines] = true
				end
			end
		end
	end

	-- THERE IS LITERALLY NO WAY TO MAKE IT PAST ALL OF THESE LINES WITHOUT CHEATING
	for i=1, 1000 do
		local garbage = {}
		for i=1, 20 do
			table.insert(garbage, string.char(love.math.random(32, 126)))
		end
		table.insert(self.lines, table.concat(garbage))
	end
end

function Typer:getCurrentLine()
	return self.lines[self.currentLine] or ""
end

function Typer:getCurrentCharacter()
	return self:getCurrentLine():sub(self.currentLetter, self.currentLetter)
end

function Typer:advanceCharacter()
	local line = self:getCurrentLine()
	local newLine = false
	repeat
		self.currentLetter = self.currentLetter + 1
		if self.currentLetter > #line then
			if self.evil[self.currentLine] then
				self.flashColor = { 255, 0, 0 }
			else
				self.flashColor = { 255, 255, 255 }
			end
			self.currentLine = self.currentLine + 1
			self.currentLetter = 1
			newLine = true
		end
	until self:getCurrentCharacter() ~= ' '
	return newLine
end

-- returns true when the word was successfully typed
function Typer:textInput(char) -- THE INCONSISTENCY HURTS BUT CAMELCASE MUST BE OBEYED
	local newLine = false

	if char:lower() == self:getCurrentCharacter() then
		newLine = self:advanceCharacter()
		if newLine then
			self.flash = 0.1
		end
	end

	return newLine
end

function Typer:update(dt)
	self.flash = self.flash - 0.2*dt
end

function Typer:drawLine(line, y, currentLetter, lineNumber)
	local font = self.font
	local spacing = 0 -- spacing between letters

	local totalWidth = font:getWidth(line) + spacing * (#line - 1)
	local currentX = Window.gameWidth/2 - totalWidth/2

	love.graphics.setFont(self.font)

	for i=1, #line do
		local char = line:sub(i,i)

		local r, g, b, alpha
		if self.evil[lineNumber] then
			r, g, b = unpack(Colors.evilText)
		elseif self.remorse[lineNumber] then
			r, g, b = unpack(Colors.remorse)
		else
			r, g, b = unpack(Colors.text)
		end

		if i >= currentLetter then
			-- checking if the letter is still untyped
			alpha = 255
		else 
			-- gray out typed letters
			alpha = 150
		end

		love.graphics.push()

		if self.evil[lineNumber] then
			love.graphics.translate(random(-1, 1), random(-1, 1))
		end

		love.graphics.setColor(r, g, b, alpha)
		love.graphics.print(char, currentX, y)
		love.graphics.pop()

		currentX = currentX + font:getWidth(char) + spacing
	end
end

function Typer:draw()
	local nextLine = self.lines[self.currentLine + 1]
	self:drawLine(self:getCurrentLine(), Window.gameHeight - 20, self.currentLetter, self.currentLine)
	if nextLine then
		self:drawLine(nextLine, Window.gameHeight - 12, #nextLine + 1, self.currentLine + 1)
	end

	if self.flash > 0 then
		love.graphics.setColor(255, 255, 255, self.flash * 255)
		love.graphics.rectangle('fill', 0, 0, Window.gameWidth, Window.gameHeight)
	end
end
